{***************************************************************}
{                                                               }
{   TMonthCalendar component                                    }
{   Date:    Sep, 30 1996                                       }
{   Version: 0.1 bate                                           }
{   Author:  Andy Lee Yiu                                       }
{   System:  Delphi 2.0 with MS IE 3.0 or greater installed     }
{                                                               }
{   E-Mail: andylee@hkstar.com                                  }
{   This is a postcardware, if you are enjoy this product,      }
{   please send the postcard to the following address           }
{   "Flat 2D 23/F Treasury Tower, Shatin, N.T. Hong Kong"       }
{***************************************************************}
{
{
  TMonthCalendar is a customizable monthly calendar control,
  subclass the new common control shipped with MS IE 3.0, that allow users
  select a day or range of days.
  This control had four styles property
    csDayState - Set Day State which can set the day with bold
                 Doesn't work at this movement
    csMultiSelect - allow Multi-Selection, depend on NumSelectDay property
    csWeekNumbers - display week numbers in left side of calendar
    csNoToday - does not show today circle

  User can change the calendar color by the property
    property CalendarBackground: TColor
    property CalendarText: TColor
    property CalendarTitleBK: TColor
    property CalendarTitleText: TColor
    property CalendarMonthBK: TColor
    property CalendarTrailingText: TColor

  type
    TDateTimeRange = array [0..1] of TDateTime; // for multi selected from and To Date

    // This style is for the function GetCurMonthRange
    TMonthRangeStyle = (mrsVisible,       // visible portion of display
                        mrsDayState);     // above plus the grayed out parts of


  Runtime properties:
    function GetCurMonthRange(Value: TMonthRangeStyle): TDateTimeRange;
      return the TDateTimeRange Value which is the starting date and ending date
      in the Monthly calendar displayed based on the TMonthRangeStyle flag. If
      set mrsVisible that return visible portion of gisplay
      set mrsDayState that retuen above plue grayed out parts of
      value [0] = starting date
      value [1] = ending date

    property Day: TDateTime
      return the selected date if Styles is MultiSelect reture first selected date.
    property MultiDays: TDateTimeRange
      return the selected range of starting and ending date.
      value [0] = starting date
      value [1] = ending date

}

unit MonthCalEx;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ComCtrls,
  CommCtrlEx;

type
  TCalendarStyle = (csDayState,     // Set Day State which can set the day with bold
                    csMultiSelect,  // allow Multi-Selection, depend on NumSelectDay
                    csWeekNumbers,  // display week numbers in left side of calendar
                    csNoToday);     // does not show today circle
  TCalendarStyles = set of TCalendarStyle;

  // TDateTime is used for multi selection style
  // array [0] is meant "From date"
  // array [1] is meant "To date"
  TDateTimeRange = array [0..1] of TDateTime; // for multi selected from and To Date

  TMCHitTest = (htTitle, htCalendar, htTodayLink, htNext, htPrev, htNowhere,
                htTitleBK, htTitleMonth, htTitleYear, htTitleBTNNext,
                htTitleBTNPrev, htCalendarBK, htCalendarDate, htCalendarDateNext,
                htCalendarDatePrev, htCalendarDay, htCalendarWeekNum);
  TMCHitTests = set of TMCHitTest;

  // This style is for the function GetCurMonthRange
  TMonthRangeStyle = (mrsVisible,       // visible portion of display
                      mrsDayState);     // above plus the grayed out parts of


  TMonthCalender = class(TWinControl)
  private
    { Private declarations }
    FCalendarBackground: TColor;
    FCalendarText: TColor;
    FCalendarTitleBK: TColor;
    FCalendarTitleText: TColor;
    FCalendarMonthBK: TColor;
    FCalendarTrailingText: TColor;
    FNumSelectDay: Integer; // the maximum number of selectable days allowed
    FDateRange: TDateTimeRange;
    FStyles: TCalendarStyles;
    FOnChange: TNotifyEvent;
    function GetMultiSelectedDay: TDateTimeRange;
    function GetSelectedDay: TDateTime;
    procedure SetCalendarBackground(Value: TColor);
    procedure SetCalendarText(Value: TColor);
    procedure SetCalendarTitleBK(Value: TColor);
    procedure SetCalendarTitleText(Value: TColor);
    procedure SetCalendarMonthBK(Value: TColor);
    procedure SetCalendarTrailingText(Value: TColor);
    procedure SetNumSelectedDay(Value: Integer);
    procedure SetMultiSelectedDay(Value: TDateTimeRange);
    procedure SetSelectedDay(Value: TDateTime);
    procedure SetStyles(Value: TCalendarStyles);
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
  protected
    { Protected declarations }
    procedure Change; dynamic;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetHitTestInfoAt(X, Y: Integer): TMCHitTests;
    function GetCurMonthRange(Value: TMonthRangeStyle): TDateTimeRange;
    property Day: TDateTime read GetSelectedDay write SetSelectedDay;
    property MultiDays: TDateTimeRange read GetMultiSelectedDay write SetMultiSelectedDay;
  published
    { Published declarations }
    property CalendarBackground: TColor read FCalendarBackground write SetCalendarBackground default clNone;
    property CalendarText: TColor read FCalendarText write SetCalendarText default clBlack;
    property CalendarTitleBK: TColor read FCalendarTitleBK write SetCalendarTitleBK default clNavy;
    property CalendarTitleText: TColor read FCalendarTitleText write SetCalendarTitleText default clWhite;
    property CalendarMonthBK: TColor read FCalendarMonthBK write SetCalendarMonthBK default clWhite;
    property CalendarTrailingText: TColor read FCalendarTrailingText write SetCalendarTrailingText default clBtnShadow;
    property Styles: TCalendarStyles read FStyles write SetStyles default [];
    property NumSelectDay: Integer read FNumSelectDay write SetNumSelectedDay default 1;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnMouseDown;
  end;

procedure Register;

implementation

constructor TMonthCalender.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csFixedWidth, csFixedHeight];
  FCalendarBackground := clNone;
  FCalendarText :=  clBlack;
  FCalendarTitleBK := clNavy;
  FCalendarTitleText := clWhite;
  FCalendarMonthBK := clWhite;
  FCalendarTrailingText := clBtnShadow;
  FStyles := [];
  FNumSelectDay := 1;
  Width := 176;
  Height := 153;
end;

destructor TMonthCalender.Destroy;
begin
  inherited Destroy;
end;

procedure TMonthCalender.CreateParams(var Params: TCreateParams);
var
  ICCEX: TInitCommonControlsEx;
begin
  ICCEX.dwSize := sizeof(TInitCommonControlsEx);
  ICCEX.dwICC := ICC_DATE_CLASSES;
  InitCommonControlsEx(@ICCEX);
  inherited CreateParams(Params);
  CreateSubClass(Params, MONTHCAL_CLASS);
  with Params do begin
    if csDayState in FStyles then Style := Style or MCS_DAYSTATE;
    if csMultiSelect in FStyles then Style := Style or MCS_MULTISELECT;
    if csWeekNumbers in FStyles then Style := Style or MCS_WEEKNUMBERS;
    if csNoToDay in FStyles then Style := Style or MCS_NOTODAY;
  end;
end;

procedure TMonthCalender.CreateWnd;
var
  Rect: TRect;
begin
  inherited CreateWnd;
  if csMultiSelect in FStyles then MonthCal_SetMaxSelCount(Handle, FNumSelectDay);
  MonthCal_SetColor(Handle, MCSC_BACKGROUND, FCalendarBackground);
  MonthCal_SetColor(Handle, MCSC_TEXT, FCalendarText);
  MonthCal_SetColor(Handle, MCSC_TITLEBK, FCalendarTitleBK);
  MonthCal_SetColor(Handle, MCSC_TITLETEXT, FCalendarTitleText);
  MonthCal_SetColor(Handle, MCSC_MONTHBK, FCalendarMonthBK);
  MonthCal_SetColor(Handle, MCSC_TRAILINGTEXT, FCalendarTrailingText);
  MonthCal_GetMinReqRect(Handle, Rect);
  Width := Rect.Right;
  Height := Rect.Bottom;
end;

function TMonthCalender.GetSelectedDay: TDateTime;
var
  SystemTime: TSYSTEMTIME;
  MultiTime: array [0..1] of TSYSTEMTIME;
begin
  if csMultiSelect in FStyles then begin
    MonthCal_GetSelRange(Handle, MultiTime);
    Result := ConvertSystemTimeToDataTime(MultiTime[0]);
  end else begin
    MonthCal_GetCurSel(Handle, SystemTime);
    Result := ConvertSystemTimeToDataTime(SystemTime);
  end;
end;

procedure TMonthCalender.SetSelectedDay(Value: TDateTime);
var
  SystemTime: TSYSTEMTIME;
begin
  SystemTime := ConvertDateTimeToSystemTime(Value);
  MonthCal_SetCurSel(Handle, SystemTime);
end;

function TMonthCalender.GetMultiSelectedDay: TDateTimeRange;
var
  SystemTime: TSYSTEMTIME;
  MultiTime: array [0..1] of TSYSTEMTIME;
begin
  if csMultiSelect in FStyles then begin
    MonthCal_GetSelRange(Handle, MultiTime);
    Result[0] := ConvertSystemTimeToDataTime(MultiTime[0]);
    Result[1] := ConvertSystemTimeToDataTime(MultiTime[1]);
  end else begin
    MonthCal_GetCurSel(Handle, SystemTime);
    Result[0] := ConvertSystemTimeToDataTime(SystemTime);
    Result[1] := ConvertSystemTimeToDataTime(SystemTime);
  end;
end;

function TMonthCalender.GetCurMonthRange(Value: TMonthRangeStyle): TDateTimeRange;
var
  SystemTime: array [0..1] of TSYSTEMTIME;
  Style: Integer;
begin
  case Value of
    mrsVisible: begin
                  Style := GMR_VISIBLE;
                end;
    mrsDayState: begin
                  Style := GMR_DAYSTATE;
                 end;
  end;
  MonthCal_GetMonthRange(Handle, Style, SystemTime);
  result[0] := ConvertSystemTimeToDataTime(SystemTime[0]);
  result[1] := ConvertSystemTimeToDataTime(SystemTime[1]);
end;

procedure TMonthCalender.SetCalendarBackground(Value: TColor);
begin
  if Value <> FCalendarBackground then begin
    FCalendarBackground := Value;
    MonthCal_SetColor(Handle, MCSC_BACKGROUND, FCalendarBackground);
  end;
end;

procedure TMonthCalender.SetCalendarText(Value: TColor);
begin
  if Value <> FCalendarText then begin
    FCalendarText := Value;
    MonthCal_SetColor(Handle, MCSC_TEXT, FCalendarText);
  end;
end;

procedure TMonthCalender.SetCalendarTitleBK(Value: TColor);
begin
  if Value <> FCalendarTitleBK then begin
    FCalendarTitleBK := Value;
    MonthCal_SetColor(Handle, MCSC_TITLEBK, FCalendarTitleBK);
  end;
end;

procedure TMonthCalender.SetCalendarTitleText(Value: TColor);
begin
  if Value <> FCalendarTitleText then begin
    FCalendarTitleText := Value;
    MonthCal_SetColor(Handle, MCSC_TITLETEXT, FCalendarTitleText);
  end;
end;

procedure TMonthCalender.SetCalendarMonthBK(Value: TColor);
begin
  if Value <> FCalendarMonthBK then begin
    FCalendarMonthBK := Value;
    MonthCal_SetColor(Handle, MCSC_MONTHBK, FCalendarMonthBK);
  end;
end;

procedure TMonthCalender.SetCalendarTrailingText(Value: TColor);
begin
  if Value <> FCalendarTrailingText then begin
    FCalendarTrailingText := Value;
    MonthCal_SetColor(Handle, MCSC_TRAILINGTEXT, FCalendarTrailingText);
  end;
end;

procedure TMonthCalender.SetMultiSelectedDay(Value: TDateTimeRange);
var
  SystemTime: array [0..1] of TSYSTEMTIME;
begin
  SystemTime[0] := ConvertDateTimeToSystemTime(Value[0]);
  SystemTime[1] := ConvertDateTimeToSystemTime(Value[1]);
  MonthCal_SetSelRange(Handle, SystemTime);
end;

procedure TMonthCalender.SetStyles(Value: TCalendarStyles);
begin
  if Value <> FStyles then begin
    FStyles := Value;
    RecreateWnd;
  end;
end;

procedure TMonthCalender.SetNumSelectedDay(Value: Integer);
begin
  if Value <> FNumSelectDay then begin
    FNumSelectDay := Value;
    RecreateWnd;
  end;
end;

procedure TMonthCalender.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;


function TMonthCalender.GetHitTestInfoAt(X, Y: Integer): TMCHitTests;
var
  HitTest: TMCHitTestInfo;
begin
  Result := [];
  with HitTest do
  begin
    cbSize := Sizeof(TMCHitTestInfo);
    pt.X := X;
    pt.Y := Y;
    uHit := $FFFFFFFF;
    MonthCal_HitTest(Handle, HitTest);
    if (uHit and MCHT_TITLE) <> 0 then
      Include(Result, htTitle);
    if (uHit and MCHT_CALENDAR) <> 0 then
      Include(Result, htCalendar);
    if (uHit and MCHT_TODAYLINK) <> 0 then
      Include(Result, htTodayLink);
    if (uHit and MCHT_NEXT) <> 0 then
      Include(Result, htNext);
    if (uHit and MCHT_PREV) <> 0 then
      Include(Result, htPrev);
    if (uHit and MCHT_NOWHERE) <> 0 then
      Include(Result, htNowhere);
    if (uHit and MCHT_TITLEBK) <> 0 then
      Include(Result, htTitleBK);
    if (uHit and MCHT_TITLEMONTH) <> 0 then
      Include(Result, htTitleMonth);
    if (uHit and MCHT_TITLEYEAR) <> 0 then
      Include(Result, htTitleYear);
    if (uHit and MCHT_TITLEBTNNEXT) <> 0 then
      Include(Result, htTitleBTNNext);
    if (uHit and MCHT_TITLEBTNPREV) <> 0 then
      Include(Result, htTitleBTNPrev);
    if (uHit and MCHT_CALENDARBK) <> 0 then
      Include(Result, htCalendarBK);
    if (uHit and MCHT_CALENDARDATE) <> 0 then
      Include(Result, htCalendarDate);
    if (uHit and MCHT_CALENDARDATENEXT) <> 0 then
      Include(Result, htCalendarDateNext);
    if (uHit and MCHT_CALENDARDATEPREV) <> 0 then
      Include(Result, htCalendarDatePrev);
    if (uHit and MCHT_CALENDARDAY) <> 0 then
      Include(Result, htCalendarDay);
    if (uHit and MCHT_CALENDARWEEKNUM) <> 0 then
      Include(Result, htCalendarWeekNum);
  end;
end;

procedure TMonthCalender.CNNotify(var Message: TWMNotify);
begin
  with Message.NMHdr^ do
    case Code of
      MCN_SELCHANGE: begin
                       Change;
                     end;
      MCN_GETDAYSTATE: begin
                       end;
    end;
end;


procedure Register;
begin
  RegisterComponents('Win95', [TMonthCalender]);
end;

end.
